@@ -5,7 +5,7 @@ use std::fmt::Debug;
 // encoded solution
 #[derive(Clone, Debug)]
 pub struct Solution<T> {
-    bits: Vec<T>,
+    pub bits: Vec<T>,
 }
 
 pub struct AnnealState<T> {
@@ -15,10 +15,10 @@ pub struct AnnealState<T> {
 
 #[derive(Debug, PartialEq)]
 pub struct AnnealerConfig {
-    alpha: f32,
-    temperature_zero: f32,
-    temperature_end: f32,
-    iteration: usize,
+    pub alpha: f32,
+    pub temperature_zero: f32,
+    pub temperature_end: f32,
+    pub iteration: usize,
 }
 
 pub struct Annealer<T, F> {
@@ -56,16 +56,16 @@ where
         self.temperature *= alpha;
     }
     // possibility to acceptance neighbor solution
-    fn acceptance<O: PartialOrd + Into<f32> + Sub<Output = O>>(
+    fn acceptance<O: PartialOrd + Into<f64> + Sub<Output = O>>(
         &mut self,
         fitness: &dyn Fn(&Solution<T>) -> O,
     ) {
         let neighbor = self.solution.neighbor();
-        let delta: f32 = (fitness(&neighbor) - fitness(&self.solution)).into();
+        let delta: f64 = (fitness(&neighbor) - fitness(&self.solution)).into();
         if delta < 0.0 {
             self.solution = neighbor;
         } else {
-            if fastrand::f32() < (-delta.abs() / self.temperature).exp() {
+            if fastrand::f64() < (-delta.abs() / self.temperature as f64).exp() {
                 self.solution = neighbor;
             }
         }
@@ -75,10 +75,10 @@ where
 impl<T, F, O> Annealer<T, F>
 where
     F: Fn(&Solution<T>) -> O,
-    O: PartialOrd + Into<f32> + Sub + Sub<Output = O>,
+    O: PartialOrd + Into<f64> + Sub + Sub<Output = O>,
     T: Copy + Debug,
 {
-    fn init<R: Fn() -> Solution<T>>(config: AnnealerConfig, fitness: F, randness: R) -> Self {
+    pub fn init<R: Fn() -> Solution<T>>(config: AnnealerConfig, fitness: F, randness: R) -> Self {
         let state = AnnealState::initial_random_state(randness, config.temperature_zero);
         Annealer {
             config,
@@ -86,7 +86,7 @@ where
             fitness,
         }
     }
-    fn anneal(&mut self) -> Solution<T> {
+    pub fn anneal(&mut self) -> Solution<T> {
         for _ in 0..self.config.iteration {
             if self.state.temperature >= self.config.temperature_end {
                 self.state.acceptance(&self.fitness);
